@@ -531,7 +531,6 @@ def get_rpc_node(node_ident):
     # as a UUID.
     if uuidutils.is_uuid_like(node_ident):
         return objects.Node.get_by_uuid(api.request.context, node_ident)
-
     # We can refer to nodes by their name, if the client supports it
     if allow_node_logical_names():
         if is_valid_logical_name(node_ident):
@@ -807,6 +806,9 @@ VERSIONED_FIELDS = {
     'network_data': versions.MINOR_66_NODE_NETWORK_DATA,
     'boot_mode': versions.MINOR_75_NODE_BOOT_MODE,
     'secure_boot': versions.MINOR_75_NODE_BOOT_MODE,
+    'shard': versions.MINOR_82_NODE_SHARD,
+    'parent_node': versions.MINOR_83_PARENT_CHILD_NODES,
+    'firmware_interface': versions.MINOR_86_FIRMWARE_INTERFACE
 }
 
 for field in V31_FIELDS:
@@ -1065,6 +1067,33 @@ def check_allow_filter_by_lessee(lessee):
              'opr': versions.MINOR_65_NODE_LESSEE})
 
 
+def check_allow_filter_by_shard(shard):
+    """Check if filtering nodes by shard is allowed.
+
+    Version 1.82 of the API allows filtering nodes by shard.
+    """
+    if (shard is not None and api.request.version.minor
+            < versions.MINOR_82_NODE_SHARD):
+        raise exception.NotAcceptable(_(
+            "Request not acceptable. The minimal required API version "
+            "should be %(base)s.%(opr)s") %
+            {'base': versions.BASE_VERSION,
+             'opr': versions.MINOR_82_NODE_SHARD})
+
+
+def check_allow_child_node_params(include_children=None,
+                                  parent_node=None):
+    if ((include_children is not None
+         or parent_node is not None)
+            and api.request.version.minor
+            < versions.MINOR_83_PARENT_CHILD_NODES):
+        raise exception.NotAcceptable(_(
+            "Request not acceptable. The minimal required API version "
+            "should be %(base)s.%(opr)s") %
+            {'base': versions.BASE_VERSION,
+             'opr': versions.MINOR_83_PARENT_CHILD_NODES})
+
+
 def initial_node_provision_state():
     """Return node state to use by default when creating new nodes.
 
@@ -1923,6 +1952,11 @@ def allow_status_in_heartbeat():
     return api.request.version.minor >= versions.MINOR_72_HEARTBEAT_STATUS
 
 
+def allow_unhold_verb():
+    """Check if the unhold verb may be passed to the API"""
+    return api.request.version.minor >= versions.MINOR_85_UNHOLD_VERB
+
+
 def check_allow_deploy_steps(target, deploy_steps):
     """Check if deploy steps are allowed"""
 
@@ -1953,3 +1987,31 @@ def check_allow_clean_disable_ramdisk(target, disable_ramdisk):
     elif target != "clean":
         raise exception.BadRequest(
             _("disable_ramdisk is supported only with manual cleaning"))
+
+
+def allow_shards_endpoint():
+    """Check if shards endpoint is available."""
+    return api.request.version.minor >= versions.MINOR_82_NODE_SHARD
+
+
+def new_continue_inspection_endpoint():
+    """Check if /v1/continue_inspection endpoint is explicitly requested."""
+    return api.request.version.minor >= versions.MINOR_84_CONTINUE_INSPECTION
+
+
+def allow_continue_inspection_endpoint():
+    """Check if /v1/continue_inspection endpoint is available.
+
+    As a special exception, we allow it in the base version so that the API
+    can be used as a drop-in replacement for the Inspector's API.
+    """
+    return (new_continue_inspection_endpoint()
+            or api.request.version.minor == versions.MINOR_1_INITIAL_VERSION)
+
+
+def allow_firmware_interface():
+    """Check if we should support firmware interface and endpoints.
+
+    Version 1.84 of the API added support for firmware interface.
+    """
+    return api.request.version.minor >= versions.MINOR_86_FIRMWARE_INTERFACE
