@@ -26,6 +26,7 @@ import hashlib
 import ipaddress
 import os
 import re
+import shlex
 import shutil
 import tempfile
 import time
@@ -56,6 +57,8 @@ TZ_RE = r'((?P<tz_sign>[+-])(?P<tz_hour>\d{2}):(?P<tz_min>\d{2}))' + \
 DATETIME_RE = re.compile(
     '%sT%s(%s)?' % (DATE_RE, TIME_RE, TZ_RE))
 
+USING_SQLITE = None
+
 
 def _get_root_helper():
     # NOTE(jlvillal): This function has been moved to ironic-lib. And is
@@ -592,7 +595,7 @@ def _get_mb_ram_available():
     return psutil.virtual_memory().available / 1024 / 1024
 
 
-def is_memory_insufficent(raise_if_fail=False):
+def is_memory_insufficient(raise_if_fail=False):
     """Checks available system memory and holds the deployment process.
 
     Evaluates the current system memory available, meaning can be
@@ -601,15 +604,15 @@ def is_memory_insufficent(raise_if_fail=False):
     or until it has timed out.
 
     This method will issue a sleep, if the amount of available memory is
-    insufficent. This is configured using the
+    insufficient. This is configured using the
     ``[DEFAULT]minimum_memory_wait_time`` and the
     ``[DEFAULT]minimum_memory_wait_retries``.
 
     :param raise_if_fail: Default False, but if set to true an
-                          InsufficentMemory exception is raised
-                          upon insufficent memory.
+                          InsufficientMemory exception is raised
+                          upon insufficient memory.
     :returns: True if the check has timed out. Otherwise None is returned.
-    :raises: InsufficentMemory if the raise_if_fail parameter is set to
+    :raises: InsufficientMemory if the raise_if_fail parameter is set to
              True.
     """
     required_memory = CONF.minimum_required_memory
@@ -633,7 +636,7 @@ def is_memory_insufficent(raise_if_fail=False):
                       'exceeded retries.',
                       log_values)
             if raise_if_fail:
-                raise exception.InsufficentMemory(
+                raise exception.InsufficientMemory(
                     free=_get_mb_ram_available(),
                     required=required_memory)
             return True
@@ -681,3 +684,57 @@ def is_fips_enabled():
     except Exception:
         pass
     return False
+
+
+def stop_after_retries(option, group=None):
+    """A tenacity retry helper that stops after retries specified in conf."""
+    # NOTE(dtantsur): fetch the option inside of the nested call, otherwise it
+    # cannot be changed in runtime.
+    def should_stop(retry_state):
+        if group:
+            conf = getattr(CONF, group)
+        else:
+            conf = CONF
+        num_retries = getattr(conf, option)
+        return retry_state.attempt_number >= num_retries + 1
+
+    return should_stop
+
+
+def is_loopback(hostname_or_ip):
+    """Check if the provided hostname or IP address is a loopback."""
+    try:
+        return ipaddress.ip_address(hostname_or_ip).is_loopback
+    except ValueError:  # host name
+        return hostname_or_ip in ('localhost', 'localhost.localdomain')
+
+
+def parse_kernel_params(params):
+    """Parse kernel parameters into a dictionary.
+
+    ``None`` is used as a value for parameters that are not in
+    the ``key=value`` format.
+
+    :param params: kernel parameters as a space-delimited string.
+    """
+    result = {}
+    for s in shlex.split(params):
+        try:
+            key, value = s.split('=', 1)
+        except ValueError:
+            result[s] = None
+        else:
+            result[key] = value
+    return result
+
+
+def is_ironic_using_sqlite():
+    """Return True if Ironic is configured to use SQLite"""
+    global USING_SQLITE
+    if USING_SQLITE is not None:
+        return USING_SQLITE
+
+    # We're being called for the first time, lets cache and
+    # return the value.
+    USING_SQLITE = 'sqlite' in CONF.database.connection.lower()
+    return USING_SQLITE
