@@ -134,6 +134,8 @@ class NodeBase(Base):
         Index('reservation_idx', 'reservation'),
         Index('conductor_group_idx', 'conductor_group'),
         Index('resource_class_idx', 'resource_class'),
+        Index('shard_idx', 'shard'),
+        Index('parent_node_idx', 'parent_node'),
         table_args())
     id = Column(Integer, primary_key=True)
     uuid = Column(String(36))
@@ -198,6 +200,7 @@ class NodeBase(Base):
     boot_interface = Column(String(255), nullable=True)
     console_interface = Column(String(255), nullable=True)
     deploy_interface = Column(String(255), nullable=True)
+    firmware_interface = Column(String(255), nullable=True)
     inspect_interface = Column(String(255), nullable=True)
     management_interface = Column(String(255), nullable=True)
     network_interface = Column(String(255), nullable=True)
@@ -210,9 +213,10 @@ class NodeBase(Base):
     storage_interface = Column(String(255), nullable=True)
     power_interface = Column(String(255), nullable=True)
     vendor_interface = Column(String(255), nullable=True)
-
     boot_mode = Column(String(16), nullable=True)
     secure_boot = Column(Boolean, nullable=True)
+    shard = Column(String(255), nullable=True)
+    parent_node = Column(String(36), nullable=True)
 
 
 class Node(NodeBase):
@@ -293,6 +297,15 @@ class Portgroup(Base):
     mode = Column(String(255))
     properties = Column(db_types.JsonEncodedDict)
 
+    _node_uuid = orm.relationship(
+        "Node",
+        viewonly=True,
+        primaryjoin="(Node.id == Portgroup.node_id)",
+        lazy="selectin",
+    )
+    node_uuid = association_proxy(
+        "_node_uuid", "uuid", creator=lambda _i: Node(uuid=_i))
+
 
 class NodeTag(Base):
     """Represents a tag of a bare metal node."""
@@ -399,7 +412,7 @@ class Allocation(Base):
     id = Column(Integer, primary_key=True)
     uuid = Column(String(36), nullable=False)
     name = Column(String(255), nullable=True)
-    node_id = Column(Integer, ForeignKey('nodes.id'), nullable=True)
+    node_id = Column(Integer, nullable=True)
     state = Column(String(15), nullable=False)
     owner = Column(String(255), nullable=True)
     last_error = Column(Text, nullable=True)
@@ -408,8 +421,7 @@ class Allocation(Base):
     candidate_nodes = Column(db_types.JsonEncodedList)
     extra = Column(db_types.JsonEncodedDict)
     # The last conductor to handle this allocation (internal field).
-    conductor_affinity = Column(Integer, ForeignKey('conductors.id'),
-                                nullable=True)
+    conductor_affinity = Column(Integer, nullable=True)
 
 
 class DeployTemplate(Base):
@@ -487,6 +499,22 @@ class NodeInventory(Base):
     node_id = Column(Integer, ForeignKey('nodes.id'), nullable=True)
 
 
+class FirmwareComponent(Base):
+    """Represents the firmware information of a bare metal node."""
+    __tablename__ = "firmware_information"
+    __table_args__ = (
+        schema.UniqueConstraint(
+            'node_id', 'component',
+            name='uniq_nodecomponent0node_id0component'),
+        table_args())
+    id = Column(Integer, primary_key=True)
+    node_id = Column(Integer, ForeignKey('nodes.id'), nullable=False)
+    component = Column(String(255), nullable=False)
+    initial_version = Column(String(255), nullable=False)
+    current_version = Column(String(255), nullable=True)
+    last_version_flashed = Column(String(255), nullable=True)
+
+
 def get_class(model_name):
     """Returns the model class with the specified name.
 
