@@ -21,7 +21,6 @@ import collections
 from ironic_lib import metrics_utils
 from oslo_log import log
 from oslo_utils import strutils
-from oslo_utils import timeutils
 import tenacity
 
 from ironic.common import boot_devices
@@ -31,6 +30,8 @@ from ironic.common.i18n import _
 from ironic.common import image_service
 from ironic.common import states
 from ironic.common import utils
+from ironic.conductor import cleaning
+from ironic.conductor import deployments
 from ironic.conductor import steps as conductor_steps
 from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
@@ -83,23 +84,26 @@ VENDOR_PROPERTIES = {
 }
 
 __HEARTBEAT_RECORD_ONLY = (states.ENROLL, states.MANAGEABLE, states.AVAILABLE,
-                           states.CLEANING, states.DEPLOYING, states.RESCUING)
+                           states.CLEANING, states.DEPLOYING, states.RESCUING,
+                           states.DEPLOYHOLD, states.CLEANHOLD)
 _HEARTBEAT_RECORD_ONLY = frozenset(__HEARTBEAT_RECORD_ONLY)
 
 _HEARTBEAT_ALLOWED = (states.DEPLOYWAIT, states.CLEANWAIT, states.RESCUEWAIT,
                       # These are allowed but don't cause any actions since
                       # they're also in HEARTBEAT_RECORD_ONLY.
-                      states.DEPLOYING, states.CLEANING, states.RESCUING)
+                      states.DEPLOYING, states.CLEANING, states.RESCUING,
+                      states.DEPLOYHOLD, states.CLEANHOLD)
 HEARTBEAT_ALLOWED = frozenset(_HEARTBEAT_ALLOWED)
 
 _FASTTRACK_HEARTBEAT_ALLOWED = (states.DEPLOYWAIT, states.CLEANWAIT,
                                 states.RESCUEWAIT, states.ENROLL,
                                 states.MANAGEABLE, states.AVAILABLE,
-                                states.DEPLOYING)
+                                states.DEPLOYING, states.CLEANHOLD,
+                                states.DEPLOYHOLD)
 FASTTRACK_HEARTBEAT_ALLOWED = frozenset(_FASTTRACK_HEARTBEAT_ALLOWED)
 
 
-@METRICS.timer('post_clean_step_hook')
+@METRICS.timer('AgentBase.post_clean_step_hook')
 def post_clean_step_hook(interface, step):
     """Decorator method for adding a post clean step hook.
 
@@ -127,7 +131,7 @@ def post_clean_step_hook(interface, step):
     return decorator
 
 
-@METRICS.timer('post_deploy_step_hook')
+@METRICS.timer('AgentBase.post_deploy_step_hook')
 def post_deploy_step_hook(interface, step):
     """Decorator method for adding a post deploy step hook.
 
@@ -186,11 +190,9 @@ def _post_step_reboot(task, step_type):
     current_step = (task.node.clean_step if step_type == 'clean'
                     else task.node.deploy_step)
     try:
-        # NOTE(fellypefca): Call prepare_ramdisk on ensure that the
-        # baremetal node boots back into the ramdisk after reboot.
-        deploy_opts = deploy_utils.build_agent_options(task.node)
-        task.driver.boot.prepare_ramdisk(task, deploy_opts)
-        manager_utils.node_power_action(task, states.REBOOT)
+        # NOTE(fellypefca): ensure that the baremetal node boots back into
+        # the ramdisk after reboot.
+        deploy_utils.reboot_to_finish_step(task)
     except Exception as e:
         msg = (_('Reboot requested by %(type)s step %(step)s failed for '
                  'node %(node)s: %(err)s') %
@@ -209,15 +211,15 @@ def _post_step_reboot(task, step_type):
         return
 
     # Signify that we've rebooted
-    driver_internal_info = task.node.driver_internal_info
-    field = ('cleaning_reboot' if step_type == 'clean'
-             else 'deployment_reboot')
-    driver_internal_info[field] = True
-    if not driver_internal_info.get('agent_secret_token_pregenerated', False):
+    if step_type == 'clean':
+        task.node.set_driver_internal_info('cleaning_reboot', True)
+    else:
+        task.node.set_driver_internal_info('deployment_reboot', True)
+    if not task.node.driver_internal_info.get(
+            'agent_secret_token_pregenerated', False):
         # Wipes out the existing recorded token because the machine will
         # need to re-establish the token.
-        driver_internal_info.pop('agent_secret_token', None)
-    task.node.driver_internal_info = driver_internal_info
+        task.node.del_driver_internal_info('agent_secret_token')
     task.node.save()
 
 
@@ -280,7 +282,7 @@ def _get_completed_command(task, commands, step_type):
         return last_command
 
 
-@METRICS.timer('log_and_raise_deployment_error')
+@METRICS.timer('AgentBase.log_and_raise_deployment_error')
 def log_and_raise_deployment_error(task, msg, collect_logs=True, exc=None):
     """Helper method to log the error and raise exception.
 
@@ -294,6 +296,15 @@ def log_and_raise_deployment_error(task, msg, collect_logs=True, exc=None):
     """
     log_traceback = (exc is not None
                      and not isinstance(exc, exception.IronicException))
+
+    # Replicate the logic in do_next_deploy_step to prepend the current step
+    step_id = (conductor_steps.step_id(task.node.deploy_step)
+               if task.node.deploy_step else None)
+    if step_id and step_id not in msg:
+        msg = _("Deploy step %(step)s failed: %(err)s") % {
+            'step': step_id, 'err': msg
+        }
+
     LOG.error(msg, exc_info=log_traceback)
     deploy_utils.set_failed_state(task, msg, collect_logs=collect_logs)
     raise exception.InstanceDeployFailure(msg)
@@ -393,6 +404,15 @@ def _step_failure_handler(task, msg, step_type, traceback=False):
         manager_utils.deploying_error_handler(task, msg, traceback=traceback)
 
 
+def _continue_steps(task, step_type):
+    if step_type == 'clean':
+        task.resume_cleaning()
+        cleaning.continue_node_clean(task)
+    else:
+        task.process_event('resume')
+        deployments.continue_node_deploy(task)
+
+
 class HeartbeatMixin(object):
     """Mixin class implementing heartbeat processing."""
 
@@ -433,12 +453,11 @@ class HeartbeatMixin(object):
         """
         return self.process_next_step(task, 'clean')
 
-    @property
-    def heartbeat_allowed_states(self):
-        """Define node states where heartbeating is allowed"""
-        if CONF.deploy.fast_track:
-            return FASTTRACK_HEARTBEAT_ALLOWED
-        return HEARTBEAT_ALLOWED
+    def heartbeat_allowed(self, node):
+        if utils.fast_track_enabled(node):
+            return node.provision_state in FASTTRACK_HEARTBEAT_ALLOWED
+        else:
+            return node.provision_state in HEARTBEAT_ALLOWED
 
     def _heartbeat_in_maintenance(self, task):
         node = task.node
@@ -486,7 +505,7 @@ class HeartbeatMixin(object):
                 msg = _('Failed to process the next deploy step')
                 self.process_next_step(task, 'deploy')
         except Exception as e:
-            last_error = _('%(msg)s. Error: %(exc)s') % {'msg': msg, 'exc': e}
+            last_error = _('%(msg)s: %(exc)s') % {'msg': msg, 'exc': e}
             LOG.exception('Asynchronous exception for node %(node)s: %(err)s',
                           {'node': task.node.uuid, 'err': last_error})
             # Do not call the error handler is the node is already DEPLOYFAIL
@@ -506,13 +525,12 @@ class HeartbeatMixin(object):
                 LOG.debug('Node %s just booted to start %s cleaning',
                           node.uuid, kind)
                 msg = _('Node failed to start the first cleaning step')
+                task.resume_cleaning()
                 # First, cache the clean steps
                 self.refresh_clean_steps(task)
                 # Then set/verify node clean steps and start cleaning
                 conductor_steps.set_node_cleaning_steps(task)
-                # The exceptions from RPC are not possible as we using cast
-                # here
-                manager_utils.notify_conductor_resume_clean(task)
+                cleaning.continue_node_clean(task)
             else:
                 msg = _('Node failed to check cleaning progress')
                 # Check if the driver is polling for completion of a step,
@@ -522,7 +540,7 @@ class HeartbeatMixin(object):
                 if not polling:
                     self.continue_cleaning(task)
         except Exception as e:
-            last_error = _('%(msg)s. Error: %(exc)s') % {'msg': msg, 'exc': e}
+            last_error = _('%(msg)s: %(exc)s') % {'msg': msg, 'exc': e}
             log_msg = ('Asynchronous exception for node %(node)s: %(err)s' %
                        {'node': task.node.uuid, 'err': last_error})
             if node.provision_state in (states.CLEANING, states.CLEANWAIT):
@@ -534,7 +552,7 @@ class HeartbeatMixin(object):
         try:
             self._finalize_rescue(task)
         except Exception as e:
-            last_error = _('%(msg)s. Error: %(exc)s') % {'msg': msg, 'exc': e}
+            last_error = _('%(msg)s: %(exc)s') % {'msg': msg, 'exc': e}
             LOG.exception('Asynchronous exception for node %(node)s: %(err)s',
                           {'node': task.node.uuid, 'err': last_error})
             if task.node.provision_state in (states.RESCUING,
@@ -556,8 +574,8 @@ class HeartbeatMixin(object):
             agent_status
         """
         # NOTE(pas-ha) immediately skip the rest if nothing to do
-        if (task.node.provision_state not in self.heartbeat_allowed_states
-            and not manager_utils.fast_track_able(task)):
+        if (not self.heartbeat_allowed(task.node)
+                and not manager_utils.fast_track_able(task)):
             LOG.error('Heartbeat from node %(node)s in unsupported '
                       'provision state %(state)s, not taking any action.',
                       {'node': task.node.uuid,
@@ -575,22 +593,17 @@ class HeartbeatMixin(object):
         node = task.node
         LOG.debug('Heartbeat from node %s in state %s (target state %s)',
                   node.uuid, node.provision_state, node.target_provision_state)
-        driver_internal_info = node.driver_internal_info
-        driver_internal_info['agent_url'] = callback_url
-        driver_internal_info['agent_version'] = agent_version
-        # Record the last heartbeat event time in UTC, so we can make
-        # decisions about it later. Can be decoded to datetime object with:
-        # datetime.datetime.strptime(var, "%Y-%m-%d %H:%M:%S.%f")
-        driver_internal_info['agent_last_heartbeat'] = str(
-            timeutils.utcnow().isoformat())
+        node.set_driver_internal_info('agent_url', callback_url)
+        node.set_driver_internal_info('agent_version', agent_version)
+        # Record the last heartbeat event time
+        node.timestamp_driver_internal_info('agent_last_heartbeat')
         if agent_verify_ca:
-            driver_internal_info['agent_verify_ca'] = agent_verify_ca
+            node.set_driver_internal_info('agent_verify_ca', agent_verify_ca)
         if agent_status:
-            driver_internal_info['agent_status'] = agent_status
+            node.set_driver_internal_info('agent_status', agent_status)
         if agent_status_message:
-            driver_internal_info['agent_status_message'] = \
-                agent_status_message
-        node.driver_internal_info = driver_internal_info
+            node.set_driver_internal_info('agent_status_message',
+                                          agent_status_message)
         node.save()
 
         if node.provision_state in _HEARTBEAT_RECORD_ONLY:
@@ -824,13 +837,12 @@ class AgentBaseMixin(object):
                 steps[step['interface']].append(step)
 
         # Save hardware manager version, steps, and date
-        info = node.driver_internal_info
-        info['hardware_manager_version'] = agent_result[
-            'hardware_manager_version']
-        info['agent_cached_%s_steps' % step_type] = dict(steps)
-        info['agent_cached_%s_steps_refreshed' % step_type] = str(
-            timeutils.utcnow())
-        node.driver_internal_info = info
+        node.set_driver_internal_info('hardware_manager_version',
+                                      agent_result['hardware_manager_version'])
+        node.set_driver_internal_info('agent_cached_%s_steps' % step_type,
+                                      dict(steps))
+        node.timestamp_driver_internal_info(
+            'agent_cached_%s_steps_refreshed' % step_type)
         node.save()
         LOG.debug('Refreshed agent %(type)s step cache for node %(node)s: '
                   '%(steps)s', {'node': node.uuid, 'steps': steps,
@@ -880,9 +892,7 @@ class AgentBaseMixin(object):
                      'continuing from current step %(step)s.',
                      {'node': node.uuid, 'step': node.clean_step})
 
-            driver_internal_info = node.driver_internal_info
-            driver_internal_info['skip_current_clean_step'] = False
-            node.driver_internal_info = driver_internal_info
+            node.set_driver_internal_info('skip_current_clean_step', False)
             node.save()
         else:
             # Restart the process, agent must have rebooted to new version
@@ -907,7 +917,7 @@ class AgentBaseMixin(object):
                 return manager_utils.deploying_error_handler(task, msg,
                                                              traceback=True)
 
-        manager_utils.notify_conductor_resume_operation(task, step_type)
+        _continue_steps(task, step_type)
 
     @METRICS.timer('AgentBaseMixin.process_next_step')
     def process_next_step(self, task, step_type, **kwargs):
@@ -937,8 +947,7 @@ class AgentBaseMixin(object):
                      else 'deployment_reboot')
             utils.pop_node_nested_field(node, 'driver_internal_info', field)
             node.save()
-            manager_utils.notify_conductor_resume_operation(task, step_type)
-            return
+            return _continue_steps(task, step_type)
 
         current_step = (node.clean_step if step_type == 'clean'
                         else node.deploy_step)
@@ -954,12 +963,12 @@ class AgentBaseMixin(object):
             return
 
         if command.get('command_status') == 'FAILED':
-            msg = (_('Agent returned error for %(type)s step %(step)s on node '
-                     '%(node)s : %(err)s.') %
+            msg = (_('%(type)s step %(step)s failed on node %(node)s. '
+                     '%(err)s') %
                    {'node': node.uuid,
                     'err': agent_client.get_command_error(command),
-                    'step': current_step,
-                    'type': step_type})
+                    'step': conductor_steps.step_id(current_step),
+                    'type': step_type.capitalize()})
             return _step_failure_handler(task, msg, step_type)
         # NOTE(dtantsur): VERSION_MISMATCH is a new alias for
         # CLEAN_VERSION_MISMATCH, remove the old one after IPA removes it.
@@ -997,7 +1006,7 @@ class AgentBaseMixin(object):
             LOG.info('Agent on node %(node)s returned %(type)s command '
                      'success, moving to next step',
                      {'node': node.uuid, 'type': step_type})
-            manager_utils.notify_conductor_resume_operation(task, step_type)
+            _continue_steps(task, step_type)
         else:
             msg = (_('Agent returned unknown status for %(type)s step %(step)s'
                      ' on node %(node)s : %(err)s.') %
@@ -1188,7 +1197,7 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                                   'command "sync"')
                     LOG.warning(
                         'Failed to flush the file system prior to hard '
-                        'rebooting the node %(node)s. Error: %(error)s',
+                        'rebooting the node %(node)s: %(error)s',
                         {'node': node.uuid, 'error': error})
 
                 manager_utils.node_power_action(task, states.POWER_OFF)
@@ -1199,24 +1208,6 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                     'error': e})
             log_and_raise_deployment_error(task, msg, exc=e)
 
-    # TODO(dtantsur): remove in W
-    @METRICS.timer('AgentDeployMixin.reboot_and_finish_deploy')
-    def reboot_and_finish_deploy(self, task):
-        """Helper method to trigger reboot on the node and finish deploy.
-
-        This method initiates a reboot on the node. On success, it
-        marks the deploy as complete. On failure, it logs the error
-        and marks deploy as failure.
-
-        :param task: a TaskManager object containing the node
-        :raises: InstanceDeployFailure, if node reboot failed.
-        """
-        # NOTE(dtantsur): do nothing here, the new deploy steps tear_down_agent
-        # and boot_instance will be picked up and finish the deploy (even for
-        # legacy deploy interfaces without decomposed steps).
-        task.process_event('wait')
-        manager_utils.notify_conductor_resume_deploy(task)
-
     @METRICS.timer('AgentDeployMixin.prepare_instance_to_boot')
     def prepare_instance_to_boot(self, task, root_uuid, efi_sys_uuid,
                                  prep_boot_part_uuid=None):
@@ -1229,12 +1220,12 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
         """
 
         node = task.node
-        if deploy_utils.get_boot_option(node) == "local":
-            # Install the boot loader
-            self.configure_local_boot(
-                task, root_uuid=root_uuid,
-                efi_system_part_uuid=efi_sys_uuid,
-                prep_boot_part_uuid=prep_boot_part_uuid)
+        # Install the boot loader
+        self.configure_local_boot(
+            task, root_uuid=root_uuid,
+            efi_system_part_uuid=efi_sys_uuid,
+            prep_boot_part_uuid=prep_boot_part_uuid)
+
         try:
             task.driver.boot.prepare_instance(task)
         except Exception as e:
@@ -1338,20 +1329,11 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                 software_raid=software_raid
             )
             if result['command_status'] == 'FAILED':
-                if not whole_disk_image:
-                    msg = (_("Failed to install a bootloader when "
-                             "deploying node %(node)s. Error: %(error)s") %
-                           {'node': node.uuid,
-                            'error': agent_client.get_command_error(result)})
-                    log_and_raise_deployment_error(task, msg)
-                else:
-                    # Its possible the install will fail if the IPA image
-                    # has not been updated, log this and continue
-                    LOG.info('Could not install bootloader for whole disk '
-                             'image for node %(node)s, Error: %(error)s"',
-                             {'node': node.uuid,
-                              'error': agent_client.get_command_error(result)})
-                    return
+                msg = (_("Failed to install a bootloader when "
+                         "deploying node %(node)s: %(error)s") %
+                       {'node': node.uuid,
+                        'error': agent_client.get_command_error(result)})
+                log_and_raise_deployment_error(task, msg)
 
         try:
             persistent = True
@@ -1362,7 +1344,7 @@ class AgentDeployMixin(HeartbeatMixin, AgentOobStepsMixin):
                                              persistent=persistent)
         except Exception as e:
             msg = (_("Failed to change the boot device to %(boot_dev)s "
-                     "when deploying node %(node)s. Error: %(error)s") %
+                     "when deploying node %(node)s: %(error)s") %
                    {'boot_dev': boot_devices.DISK, 'node': node.uuid,
                     'error': e})
             log_and_raise_deployment_error(task, msg, exc=e)
