@@ -26,13 +26,16 @@ from ironic_lib import utils as ironic_utils
 from oslo_log import log
 
 from ironic.common import exception
+from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import images
+from ironic.common import states
 from ironic.common import swift
 from ironic.common import utils
 from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
+from ironic.drivers.modules import image_cache
 from ironic.drivers import utils as driver_utils
 
 LOG = log.getLogger(__name__)
@@ -166,7 +169,7 @@ class ImageHandler(object):
 
         return urlparse.urlunparse(parsed_url)
 
-    def publish_image(self, image_file, object_name):
+    def publish_image(self, image_file, object_name, node_http_url=None):
         """Make image file downloadable.
 
         Depending on ironic settings, pushes given file into Swift or copies
@@ -175,6 +178,9 @@ class ImageHandler(object):
 
         :param image_file: path to file to publish
         :param object_name: name of the published file
+        :param node_http_url: a url to be used to publish the image. If set,
+                              the values from external_http_url and http_url
+                              from CONF.deploy won't be used.
         :return: a URL to download published file
         """
 
@@ -190,6 +196,8 @@ class ImageHandler(object):
                                     object_headers=object_headers)
 
             image_url = swift_api.get_temp_url(container, object_name, timeout)
+            image_url = self._append_filename_param(
+                image_url, os.path.basename(image_file))
 
         else:
             public_dir = os.path.join(CONF.deploy.http_root,
@@ -203,6 +211,16 @@ class ImageHandler(object):
             try:
                 os.link(image_file, published_file)
                 os.chmod(image_file, self._file_permission)
+                try:
+                    utils.execute(
+                        '/usr/sbin/restorecon', '-i', '-R', 'v', public_dir)
+                except FileNotFoundError as exc:
+                    LOG.debug(
+                        "Could not restore SELinux context on "
+                        "%(public_dir)s, restorecon command not found.\n"
+                        "Error: %(error)s",
+                        {'public_dir': public_dir,
+                         'error': exc})
 
             except OSError as exc:
                 LOG.debug(
@@ -215,15 +233,26 @@ class ImageHandler(object):
                 shutil.copyfile(image_file, published_file)
                 os.chmod(published_file, self._file_permission)
 
-            http_url = CONF.deploy.external_http_url or CONF.deploy.http_url
+            http_url = (node_http_url or CONF.deploy.external_http_url
+                        or CONF.deploy.http_url)
             image_url = os.path.join(http_url, self._image_subdir, object_name)
 
-        image_url = self._append_filename_param(
-            image_url, os.path.basename(image_file))
-
         return image_url
 
 
+@image_cache.cleanup(priority=75)
+class ISOImageCache(image_cache.ImageCache):
+
+    def __init__(self):
+        master_path = CONF.deploy.iso_master_path or None
+        super(self.__class__, self).__init__(
+            master_path,
+            # MiB -> B
+            cache_size=CONF.deploy.iso_cache_size * 1024 * 1024,
+            # min -> sec
+            cache_ttl=CONF.deploy.iso_cache_ttl * 60)
+
+
 def _get_name(node, prefix='', suffix=''):
     """Get an object name for a given node.
 
@@ -287,8 +316,9 @@ def prepare_floppy_image(task, params=None):
         images.create_vfat_image(vfat_image_tmpfile, parameters=params)
 
         img_handler = ImageHandler(task.node.driver)
-
-        image_url = img_handler.publish_image(vfat_image_tmpfile, object_name)
+        node_http_url = task.node.driver_info.get("external_http_url")
+        image_url = img_handler.publish_image(vfat_image_tmpfile, object_name,
+                                              node_http_url)
 
     LOG.debug("Created floppy image %(name)s in Swift for node %(node)s, "
               "exposed as temporary URL "
@@ -424,71 +454,93 @@ def _prepare_iso_image(task, kernel_href, ramdisk_href,
             "building ISO, or explicit ISO for %(node)s") %
             {'node': task.node.uuid})
 
+    i_info = task.node.instance_info
+    is_ramdisk_boot = (
+        task.node.provision_state == states.DEPLOYING
+        and deploy_utils.get_boot_option(task.node) == 'ramdisk'
+    )
+
+    if is_ramdisk_boot:
+        download_source = (i_info.get('ramdisk_image_download_source')
+                           or CONF.deploy.ramdisk_image_download_source)
+    else:
+        download_source = CONF.deploy.ramdisk_image_download_source
+
     # NOTE(rpittau): if base_iso is defined as http address, we just access
     # it directly.
-    if base_iso and CONF.deploy.ramdisk_image_download_source == 'http':
-        if base_iso.startswith(('http://', 'https://')):
+    if base_iso:
+        scheme = urlparse.urlparse(base_iso).scheme.lower()
+        if scheme == 'swift':
+            # FIXME(dtantsur): iLO supports swift: scheme. In the long run we
+            # should support it for all boot interfaces by using temporary
+            # URLs. Until it's done, return base_iso as it is.
             return base_iso
-        LOG.debug("ramdisk_image_download_source set to http but "
-                  "boot_iso is not an HTTP URL: %(boot_iso)s",
-                  {"boot_iso": base_iso})
+
+        if (download_source == 'swift'
+                and service_utils.is_glance_image(base_iso)):
+            base_iso = (
+                images.get_temp_url_for_glance_image(task.context, base_iso))
+            # get_temp_url_for_glance_image return an HTTP (or HTTPS - doesn't
+            # matter here) image.
+            scheme = 'http'
+
+        if download_source != 'local':
+            if scheme in ('http', 'https'):
+                return base_iso
+            LOG.debug("ramdisk_image_download_source set to http but "
+                      "boot_iso is not an HTTP URL: %(boot_iso)s",
+                      {"boot_iso": base_iso})
 
     img_handler = ImageHandler(task.node.driver)
 
-    i_info = task.node.instance_info
-
-    # NOTE(TheJulia): Until we support modifying a base iso, most of
-    # this logic actually does nothing in the end. But it should!
-    if deploy_utils.get_boot_option(task.node) == "ramdisk":
-        if not base_iso:
-            kernel_params = "root=/dev/ram0 text "
-            kernel_params += i_info.get("ramdisk_kernel_arguments", "")
-        else:
-            kernel_params = None
-
-    else:
-        kernel_params = driver_utils.get_kernel_append_params(
-            task.node, default=img_handler.kernel_params)
-
-    if params and not base_iso:
-        kernel_params = ' '.join(
-            (kernel_params, ' '.join(
-                ('%s=%s' % kv) if kv[1] is not None else kv[0]
-                for kv in params.items())))
-
     boot_mode = boot_mode_utils.get_boot_mode(task.node)
 
-    if base_iso:
-        # TODO(dtantsur): fix this limitation eventually (see
-        # images.create_boot_iso).
-        LOG.debug('Using pre-built %(boot_mode)s ISO %(iso)s for node '
-                  '%(node)s, custom configuration will not be available',
-                  {'boot_mode': boot_mode, 'node': task.node.uuid,
-                   'iso': base_iso})
-    else:
-        LOG.debug("Trying to create %(boot_mode)s ISO image for node %(node)s "
-                  "with kernel %(kernel_href)s, ramdisk %(ramdisk_href)s, "
-                  "bootloader %(bootloader_href)s and kernel params %(params)s"
-                  "", {'node': task.node.uuid,
-                       'boot_mode': boot_mode,
-                       'kernel_href': kernel_href,
-                       'ramdisk_href': ramdisk_href,
-                       'bootloader_href': bootloader_href,
-                       'params': kernel_params})
+    with tempfile.TemporaryDirectory(dir=CONF.tempdir) as boot_file_dir:
 
-    with tempfile.NamedTemporaryFile(
-            dir=CONF.tempdir, suffix='.iso') as boot_fileobj:
+        boot_iso_tmp_file = os.path.join(boot_file_dir, 'boot.iso')
+        if base_iso:
+            # NOTE(dtantsur): this should be "params or inject_files", but
+            # params are always populated in the calling code.
+            log_func = LOG.warning if inject_files else LOG.debug
+            log_func('Using pre-built %(boot_mode)s ISO %(iso)s for node '
+                     '%(node)s, custom configuration will not be available',
+                     {'boot_mode': boot_mode, 'node': task.node.uuid,
+                      'iso': base_iso})
 
-        boot_iso_tmp_file = boot_fileobj.name
-        images.create_boot_iso(
-            task.context, boot_iso_tmp_file,
-            kernel_href, ramdisk_href,
-            esp_image_href=bootloader_href,
-            root_uuid=root_uuid,
-            kernel_params=kernel_params,
-            boot_mode=boot_mode,
-            base_iso=base_iso,
-            inject_files=inject_files)
+            ISOImageCache().fetch_image(base_iso, boot_iso_tmp_file,
+                                        ctx=task.context, force_raw=False)
+        else:
+            if is_ramdisk_boot:
+                kernel_params = "root=/dev/ram0 text "
+                kernel_params += i_info.get("ramdisk_kernel_arguments", "")
+            else:
+                kernel_params = driver_utils.get_kernel_append_params(
+                    task.node, default=img_handler.kernel_params)
+
+            if params:
+                kernel_params = ' '.join(
+                    (kernel_params, ' '.join(
+                        ('%s=%s' % kv) if kv[1] is not None else kv[0]
+                        for kv in params.items())))
+
+            LOG.debug(
+                "Trying to create %(boot_mode)s ISO image for node %(node)s "
+                "with kernel %(kernel_href)s, ramdisk %(ramdisk_href)s, "
+                "bootloader %(bootloader_href)s and kernel params %(params)s",
+                {'node': task.node.uuid,
+                 'boot_mode': boot_mode,
+                 'kernel_href': kernel_href,
+                 'ramdisk_href': ramdisk_href,
+                 'bootloader_href': bootloader_href,
+                 'params': kernel_params})
+            images.create_boot_iso(
+                task.context, boot_iso_tmp_file,
+                kernel_href, ramdisk_href,
+                esp_image_href=bootloader_href,
+                root_uuid=root_uuid,
+                kernel_params=kernel_params,
+                boot_mode=boot_mode,
+                inject_files=inject_files)
 
         iso_object_name = _get_name(task.node, prefix='boot', suffix='.iso')
 
