@@ -15,6 +15,7 @@
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import strutils
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from oslo_utils import versionutils
 from oslo_versionedobjects import base as object_base
@@ -28,7 +29,7 @@ from ironic.objects import base
 from ironic.objects import fields as object_fields
 from ironic.objects import notification
 
-REQUIRED_INT_PROPERTIES = ['local_gb', 'cpus', 'memory_mb']
+REQUIRED_INT_PROPERTIES = ['local_gb', 'memory_mb']
 
 CONF = cfg.CONF
 LOG = log.getLogger(__name__)
@@ -76,7 +77,11 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.33: Add retired and retired_reason fields
     # Version 1.34: Add lessee field
     # Version 1.35: Add network_data field
-    VERSION = '1.35'
+    # Version 1.36: Add boot_mode and secure_boot fields
+    # Version 1.37: Add shard field
+    # Version 1.38: Add parent_node field
+    # Version 1.39: Add firmware_interface field
+    VERSION = '1.39'
 
     dbapi = db_api.get_instance()
 
@@ -151,6 +156,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'boot_interface': object_fields.StringField(nullable=True),
         'console_interface': object_fields.StringField(nullable=True),
         'deploy_interface': object_fields.StringField(nullable=True),
+        'firmware_interface': object_fields.StringField(nullable=True),
         'inspect_interface': object_fields.StringField(nullable=True),
         'management_interface': object_fields.StringField(nullable=True),
         'network_interface': object_fields.StringField(nullable=True),
@@ -166,6 +172,10 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'retired': objects.fields.BooleanField(nullable=True),
         'retired_reason': object_fields.StringField(nullable=True),
         'network_data': object_fields.FlexibleDictField(nullable=True),
+        'boot_mode': object_fields.StringField(nullable=True),
+        'secure_boot': object_fields.BooleanField(nullable=True),
+        'shard': object_fields.StringField(nullable=True),
+        'parent_node': object_fields.StringField(nullable=True),
     }
 
     def as_dict(self, secure=False, mask_configdrive=True):
@@ -185,7 +195,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         return d
 
     def _validate_property_values(self, properties):
-        """Check if the input of local_gb, cpus and memory_mb are valid.
+        """Check if the input of local_gb and memory_mb are valid.
 
         :param properties: a dict contains the node's information.
         """
@@ -213,12 +223,14 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             raise exception.InvalidParameterValue(msg)
 
     def _set_from_db_object(self, context, db_object, fields=None):
-        fields = set(fields or self.fields) - {'traits'}
-        super(Node, self)._set_from_db_object(context, db_object, fields)
-        self.traits = object_base.obj_make_list(
-            context, objects.TraitList(context),
-            objects.Trait, db_object['traits'])
-        self.traits.obj_reset_changes()
+        use_fields = set(fields or self.fields) - {'traits'}
+        super(Node, self)._set_from_db_object(context, db_object, use_fields)
+        if not fields or 'traits' in fields:
+            self.traits = object_base.obj_make_list(
+                context, objects.TraitList(context),
+                objects.Trait, db_object['traits'],
+                fields=['trait', 'version'])
+            self.traits.obj_reset_changes()
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
     # methods can be used in the future to replace current explicit RPC calls.
@@ -313,7 +325,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # @object_base.remotable_classmethod
     @classmethod
     def list(cls, context, limit=None, marker=None, sort_key=None,
-             sort_dir=None, filters=None):
+             sort_dir=None, filters=None, fields=None):
         """Return a list of Node objects.
 
         :param cls: the :class:`Node`
@@ -323,13 +335,29 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         :param sort_key: column to sort results by.
         :param sort_dir: direction to sort. "asc" or "desc".
         :param filters: Filters to apply.
+        :param fields: Requested fields to be returned. Please note, some
+                       fields are mandatory for the data model and are
+                       automatically included. These are: id, version,
+                       updated_at, created_at, owner, and lessee.
         :returns: a list of :class:`Node` object.
-
         """
+        if fields:
+            # All requests must include version, updated_at, created_at
+            # owner, and lessee to support access controls and database
+            # version model updates. Driver and conductor_group are required
+            # for conductor mapping.
+            target_fields = ['id'] + fields[:] + ['version', 'updated_at',
+                                                  'created_at', 'owner',
+                                                  'lessee', 'driver',
+                                                  'conductor_group']
+        else:
+            target_fields = None
+
         db_nodes = cls.dbapi.get_node_list(filters=filters, limit=limit,
                                            marker=marker, sort_key=sort_key,
-                                           sort_dir=sort_dir)
-        return cls._from_db_object_list(context, db_nodes)
+                                           sort_dir=sort_dir,
+                                           fields=target_fields)
+        return cls._from_db_object_list(context, db_nodes, target_fields)
 
     # NOTE(xek): We don't want to enable RPC on this call just yet. Remotable
     # methods can be used in the future to replace current explicit RPC calls.
@@ -523,6 +551,12 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
+    def get_interface(self, iface):
+        iface_name = '%s_interface' % iface
+        impl_name = self.instance_info.get(iface_name,
+                                           getattr(self, iface_name))
+        return impl_name
+
     def _convert_deploy_step_field(self, target_version,
                                    remove_unavailable_fields=True):
         # NOTE(rloo): Typically we set the value to None. However,
@@ -626,6 +660,13 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             be set to None or removed.
         Version 1.35: network_data was added. For versions prior to this, it
             should be set to empty dict (or removed).
+        Version 1.36: boot_mode, secure_boot were was added. Defaults are None.
+            For versions prior to this, it should be set to None or removed.
+        Version 1.37: shard was added. Default is None. For versions prior to
+            this, it should be set to None or removed.
+        Version 1.39: firmware_interface field was added. Its default value is
+            None. For versions prior to this, it should be set to None (or
+            removed).
 
         :param target_version: the desired version of the object
         :param remove_unavailable_fields: True to remove fields that are
@@ -640,7 +681,9 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
                   ('bios_interface', 24), ('fault', 25),
                   ('automated_clean', 28), ('protected_reason', 29),
                   ('owner', 30), ('allocation_id', 31), ('description', 32),
-                  ('retired_reason', 33), ('lessee', 34)]
+                  ('retired_reason', 33), ('lessee', 34), ('boot_mode', 36),
+                  ('secure_boot', 36), ('shard', 37),
+                  ('firmware_interface', 39)]
 
         for name, minor in fields:
             self._adjust_field_to_version(name, None, target_version,
@@ -661,6 +704,55 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         self._convert_network_data_field(target_version,
                                          remove_unavailable_fields)
 
+    def set_driver_internal_info(self, key, value):
+        """Set a `driver_internal_info` value.
+
+        Setting a `driver_internal_info` dict value via this method
+        ensures that this field will be flagged for saving.
+
+        :param key: Key of item to set
+        :param value: Value of item to set
+        """
+        self.driver_internal_info[key] = value
+        self._changed_fields.add('driver_internal_info')
+
+    def del_driver_internal_info(self, key, default_value=None):
+        """Pop a value from the driver_internal_info.
+
+        Removing a `driver_internal_info` dict value via this method
+        ensures that this field will be flagged for saving.
+
+        :param key: Key of item to pop off the `driver_internal_info` dict
+        :param default_value: Value to return if the key doesn't exist
+        :returns: The removed value, or `default_value`
+        """
+        if key in self.driver_internal_info:
+            self._changed_fields.add('driver_internal_info')
+            return self.driver_internal_info.pop(key, default_value)
+        return default_value
+
+    def timestamp_driver_internal_info(self, key):
+        """Set a `driver_internal_info` value with the current timestamp.
+
+        Setting a `driver_internal_info` timestamp value via this method
+        ensures that this field will be flagged for saving.
+
+        :param key: Key of item to set the timestamp on
+        """
+        self.set_driver_internal_info(key, timeutils.utcnow().isoformat())
+
+    def set_instance_info(self, key, value):
+        """Set an `instance_info` value.
+
+        Setting a `instance_info` dict value via this method ensures that this
+        field will be flagged for saving.
+
+        :param key: Key of item to set
+        :param value: Value of item to set
+        """
+        self.instance_info[key] = value
+        self._changed_fields.add('instance_info')
+
 
 @base.IronicObjectRegistry.register
 class NodePayload(notification.NotificationPayloadBase):
@@ -682,6 +774,8 @@ class NodePayload(notification.NotificationPayloadBase):
         'description': ('node', 'description'),
         'driver': ('node', 'driver'),
         'extra': ('node', 'extra'),
+        'boot_mode': ('node', 'boot_mode'),
+        'secure_boot': ('node', 'secure_boot'),
         'inspection_finished_at': ('node', 'inspection_finished_at'),
         'inspection_started_at': ('node', 'inspection_started_at'),
         'instance_uuid': ('node', 'instance_uuid'),
@@ -736,7 +830,8 @@ class NodePayload(notification.NotificationPayloadBase):
     # Version 1.13: Add description field.
     # Version 1.14: Add retired and retired_reason fields exposed via API.
     # Version 1.15: Add node lessee field.
-    VERSION = '1.15'
+    # Version 1.16: Add boot_mode and secure_boot fields.
+    VERSION = '1.16'
     fields = {
         'clean_step': object_fields.FlexibleDictField(nullable=True),
         'conductor_group': object_fields.StringField(nullable=True),
@@ -746,6 +841,8 @@ class NodePayload(notification.NotificationPayloadBase):
         'description': object_fields.StringField(nullable=True),
         'driver': object_fields.StringField(nullable=True),
         'extra': object_fields.FlexibleDictField(nullable=True),
+        'boot_mode': object_fields.StringField(nullable=True),
+        'secure_boot': object_fields.BooleanField(nullable=True),
         'inspection_finished_at': object_fields.DateTimeField(nullable=True),
         'inspection_started_at': object_fields.DateTimeField(nullable=True),
         'instance_uuid': object_fields.UUIDField(nullable=True),
@@ -825,7 +922,8 @@ class NodeSetPowerStatePayload(NodePayload):
     # Version 1.13: Parent NodePayload version 1.13
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
-    VERSION = '1.15'
+    # Version 1.16: Parent NodePayload version 1.16
+    VERSION = '1.16'
 
     fields = {
         # "to_power" indicates the future target_power_state of the node. A
@@ -881,7 +979,8 @@ class NodeCorrectedPowerStatePayload(NodePayload):
     # Version 1.13: Parent NodePayload version 1.13
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
-    VERSION = '1.15'
+    # Version 1.16: Parent NodePayload version 1.16
+    VERSION = '1.16'
 
     fields = {
         'from_power': object_fields.StringField(nullable=True)
@@ -922,7 +1021,8 @@ class NodeSetProvisionStatePayload(NodePayload):
     # Version 1.14: Parent NodePayload version 1.14
     # Version 1.15: Parent NodePayload version 1.15
     # Version 1.16: add driver_internal_info
-    VERSION = '1.16'
+    # Version 1.17: Parent NodePayload version 1.16
+    VERSION = '1.17'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
@@ -971,7 +1071,8 @@ class NodeCRUDPayload(NodePayload):
     # Version 1.11: Parent NodePayload version 1.13
     # Version 1.12: Parent NodePayload version 1.14
     # Version 1.13: Parent NodePayload version 1.15
-    VERSION = '1.13'
+    # Version 1.14: Parent NodePayload version 1.16
+    VERSION = '1.14'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
