@@ -26,7 +26,10 @@ from ironic.objects import bios
 from ironic.objects import chassis
 from ironic.objects import conductor
 from ironic.objects import deploy_template
+from ironic.objects import firmware
 from ironic.objects import node
+from ironic.objects import node_history
+from ironic.objects import node_inventory
 from ironic.objects import port
 from ironic.objects import portgroup
 from ironic.objects import trait
@@ -104,7 +107,7 @@ def get_test_irmc_info():
         "irmc_address": "1.2.3.4",
         "irmc_username": "admin0",
         "irmc_password": "fake0",
-        "irmc_port": 80,
+        "irmc_port": "80",
         "irmc_auth_method": "digest",
     }
 
@@ -163,7 +166,6 @@ def get_test_snmp_info(**kw):
 def get_test_node(**kw):
     properties = {
         "cpu_arch": "x86_64",
-        "cpus": "8",
         "local_gb": "10",
         "memory_mb": "4096",
     }
@@ -235,6 +237,8 @@ def get_test_node(**kw):
         'network_data': kw.get('network_data'),
         'boot_mode': kw.get('boot_mode', None),
         'secure_boot': kw.get('secure_boot', None),
+        'shard': kw.get('shard', None),
+        'parent_node': kw.get('parent_node', None)
     }
 
     for iface in drivers_base.ALL_INTERFACES:
@@ -270,6 +274,8 @@ def get_test_port(**kw):
         'version': kw.get('version', port.Port.VERSION),
         'uuid': kw.get('uuid', '1be26c0b-03f2-4d2e-ae87-c02d7f33c781'),
         'node_id': kw.get('node_id', 123),
+        'node_uuid': kw.get('node_uuid',
+                            '59d102f7-5840-4299-8ec8-80c0ebae9de1'),
         'address': kw.get('address', '52:54:00:cf:2d:31'),
         'extra': kw.get('extra', {}),
         'created_at': kw.get('created_at'),
@@ -444,6 +450,8 @@ def get_test_portgroup(**kw):
         'uuid': kw.get('uuid', '6eb02b44-18a3-4659-8c0b-8d2802581ae4'),
         'name': kw.get('name', 'fooname'),
         'node_id': kw.get('node_id', 123),
+        'node_uuid': kw.get('node_uuid',
+                            '40481b96-306b-4a33-901f-795a3dc2f397'),
         'address': kw.get('address', '52:54:00:cf:2d:31'),
         'extra': kw.get('extra', {}),
         'created_at': kw.get('created_at'),
@@ -502,7 +510,6 @@ def create_test_node_tag(**kw):
 def get_test_xclarity_properties():
     return {
         "cpu_arch": "x86_64",
-        "cpus": "8",
         "local_gb": "10",
         "memory_mb": "4096",
     }
@@ -690,3 +697,99 @@ def get_test_ibmc_info():
         "ibmc_password": "password",
         "verify_ca": False,
     }
+
+
+def get_test_history(**kw):
+    return {
+        'id': kw.get('id', 345),
+        'version': kw.get('version', node_history.NodeHistory.VERSION),
+        'uuid': kw.get('uuid', '6f8a5d5c-0f2d-4b2c-a62a-a38e300e3f31'),
+        'node_id': kw.get('node_id', 123),
+        'event': kw.get('event', 'Something is wrong'),
+        'conductor': kw.get('conductor', 'host-1'),
+        'severity': kw.get('severity', 'ERROR'),
+        'event_type': kw.get('event_type', 'provisioning'),
+        'user': kw.get('user', 'fake-user'),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+    }
+
+
+def create_test_history(**kw):
+    """Create test history entry in DB and return NodeHistory DB object.
+
+    :param kw: kwargs with overriding values for port's attributes.
+    :returns: Test NodeHistory DB object.
+    """
+    history = get_test_history(**kw)
+    # Let DB generate ID if it isn't specified explicitly
+    if 'id' not in kw:
+        del history['id']
+    dbapi = db_api.get_instance()
+    return dbapi.create_node_history(history)
+
+
+def get_test_inventory(**kw):
+    return {
+        'id': kw.get('id', 345),
+        'version': kw.get('version', node_inventory.NodeInventory.VERSION),
+        'node_id': kw.get('node_id', 123),
+        'inventory_data': kw.get('inventory', {"inventory": "test"}),
+        'plugin_data': kw.get('plugin_data', {"pdata": {"plugin": "data"}}),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+    }
+
+
+def create_test_inventory(**kw):
+    """Create test inventory entry in DB and return NodeInventory DB object.
+
+    :param kw: kwargs with overriding values for inventory attributes.
+    :returns: Test NodeInventory DB object.
+    """
+    inventory = get_test_inventory(**kw)
+    # Let DB generate ID if it isn't specified explicitly
+    if 'id' not in kw:
+        del inventory['id']
+    dbapi = db_api.get_instance()
+    return dbapi.create_node_inventory(inventory)
+
+
+def create_test_firmware_component(**kw):
+    """Create test Firmware Component entry in DB and return object.
+
+    Function to be used to create test FirmwareComponent object in the DB.
+
+    :param kw: kwargs with overriding values for firmware component
+    :returns: Test FirmwareComponent DB object
+    """
+    fw_cmp_values = get_test_firmware_component(**kw)
+    if 'id' not in kw:
+        del fw_cmp_values['id']
+    dbapi = db_api.get_instance()
+    return dbapi.create_firmware_component(fw_cmp_values)
+
+
+def get_test_firmware_component(**kw):
+    # NOTE(iurygregory): update version to get from the object in the object
+    # patch.
+    return {
+        'id': kw.get('id', 256),
+        'node_id': kw.get('node_id', 123),
+        'component': kw.get('component', 'bmc'),
+        'initial_version': kw.get('initial_version', 'v1.0.0'),
+        'current_version': kw.get('current_version', 'v1.0.0'),
+        'last_version_flashed': kw.get('last_version_flashed', None),
+        'version': kw.get('version', firmware.FirmwareComponent.VERSION),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+    }
+
+
+def get_test_firmware_component_list():
+    return [
+        {'component': 'bmc', 'initial_version': 'v1.0.0',
+         'current_version': 'v1.0.0', 'last_version_flashed': None},
+        {'component': 'BIOS', 'initial_version': 'v1.5.0',
+         'current_version': 'v1.5.0', 'last_version_flashed': None},
+    ]
